import fs from "fs-extra";
import child from "child_process";
import path, { join } from "path";
import { get, set } from "bottom-line-utils";
import { dirname } from "./dirname/dirname.js";
const thisDir = path.dirname(path.dirname(dirname));
const run = async (command, args, options) => {
    return new Promise((resolve, reject) => {
        const proc = child.spawn(command, args, options);
        proc.stdout.on("data", (data) => {
            console.info(String(data).trim());
        });
        proc.stderr.on("data", (data) => {
            console.info(String(data).trim());
        });
        proc.on("error", (error) => {
            reject(error);
        });
        proc.on("close", (code) => {
            if (!code) {
                resolve();
                return;
            }
            reject(new Error(`${command} ${args.join(" ")} exited with code ${code}`));
        });
    });
};
const PKG_JSON_INDENT = 2;
const JSON_INDENT = 4;
class Migration {
    constructor(opts) {
        this._targetDir = opts.targetDir;
        this._pkg = opts.pkg;
    }
    _savePkg() {
        return fs.writeFile(join(this._targetDir, "package.json"), JSON.stringify(this._pkg, null, PKG_JSON_INDENT));
    }
    get jsx() {
        return Boolean(this._pkg.libraryTemplate.jsx);
    }
    get targetDir() {
        return this._targetDir;
    }
    get pkg() {
        return this._pkg;
    }
    async setPath(objPath, value) {
        set(this._pkg, objPath, value);
        await this._savePkg();
    }
    async setScript(scriptName, value) {
        await this.setPath(["scripts", scriptName], value);
    }
    async deletePath(objPath) {
        const thePath = typeof objPath === "string" ? objPath.split(".") : [...objPath];
        const last = thePath.pop();
        const item = get(this._pkg, thePath);
        if (item == null || !last) {
            return;
        }
        delete item[last];
        await this._savePkg();
    }
    async deleteScript(scriptName) {
        await this.deletePath(["scripts", scriptName]);
    }
    async upgradeScript(scriptName, oldValue, newValue) {
        this.assertScript(scriptName, oldValue, new Error("cannot update " + scriptName + " script as it was modified\n  "
            + "wanted new value:\n  `" + newValue + "`"));
        await this.setScript(scriptName, newValue);
    }
    async addDependency(name, version) {
        await this.setPath(["dependencies", name], version);
    }
    async addDevDependency(name, version) {
        await this.setPath(["devDependencies", name], version);
    }
    async upgradeDependency(name, version, defaultType = "dependencies") {
        var _a;
        const type = (_a = this.findDependency(name)) !== null && _a !== void 0 ? _a : defaultType;
        await this.setPath([type, name], version);
    }
    findDependency(name) {
        var _a, _b;
        const deps = (_a = this._pkg.dependencies) !== null && _a !== void 0 ? _a : {};
        const dev = (_b = this._pkg.devDependencies) !== null && _b !== void 0 ? _b : {};
        if (deps[name]) {
            return "dependencies";
        }
        if (dev[name]) {
            return "devDependencies";
        }
    }
    async remove(dirName) {
        await fs.remove(join(this._targetDir, dirName));
    }
    async copy(sourceName, targetName = null, overwrite = true) {
        await fs.copy(join(thisDir, sourceName), join(this._targetDir, targetName !== null && targetName !== void 0 ? targetName : sourceName), { overwrite });
    }
    async rename(source, target) {
        await fs.rename(join(this._targetDir, source), join(this._targetDir, target));
    }
    async updateContents(file, updater) {
        const target = join(this._targetDir, file);
        const data = String(await fs.readFile(target));
        const contents = await updater(data);
        await fs.writeFile(target, contents);
    }
    async updateContentsJSON(file, updater) {
        const target = join(this._targetDir, file);
        const data = JSON.parse(String(await fs.readFile(target)));
        const newData = await updater(data, set.bind(null, data));
        await fs.writeFile(target, JSON.stringify(newData !== null && newData !== void 0 ? newData : data, null, JSON_INDENT));
    }
    async updatePath(objPath, updater) {
        const current = get(this._pkg, objPath);
        const newValue = updater(String(current));
        await this.setPath(objPath, newValue);
    }
    assertPath(objPath, value, error) {
        if (get(this._pkg, objPath) !== value) {
            throw error;
        }
    }
    assertNoPath(objPath, error) {
        if (get(this._pkg, objPath) !== undefined) {
            throw error;
        }
    }
    assertScript(scriptName, value, error) {
        this.assertPath(["scripts", scriptName], value, error);
    }
    assertNoScript(scriptName, error) {
        this.assertNoPath(["scripts", scriptName], error);
    }
    assertNoDependency(name, error) {
        if (this._pkg.dependencies && name in this._pkg.dependencies) {
            throw error;
        }
    }
    assertNoDevDependency(name, error) {
        if (this._pkg.devDependencies && name in this._pkg.devDependencies) {
            throw error;
        }
    }
    assertNoAnyDependency(name, error) {
        this.assertNoDependency(name, error);
        this.assertNoDevDependency(name, error);
    }
    assertDevDependency(name, version, error) {
        if (!this._pkg.devDependencies
            || ((version && this._pkg.devDependencies[name] !== version)
                || (!version && !this._pkg.devDependencies[name]))) {
            throw error;
        }
    }
    async removeDependency(name) {
        await this.deletePath(["dependencies", name]);
    }
    async removeDevDependency(name) {
        await this.deletePath(["devDependencies", name]);
    }
    async removeAnyDependency(name) {
        const depType = this.findDependency(name);
        if (!depType) {
            return;
        }
        await this.deletePath([depType, name]);
    }
    yarn() {
        return run("yarn", [], {
            cwd: this._targetDir,
            shell: true,
        });
    }
}
export { Migration };
//# sourceMappingURL=Migration.js.map