/* eslint-disable max-lines */
import fs from "fs-extra";
import child from "child_process";
import path, { join } from "path";
import { get, set } from "bottom-line-utils";
import type { SpawnOptionsWithoutStdio } from "child_process";

import type { PackageJson } from "./types.js";
import { dirname } from "./dirname/dirname.js";

const thisDir = path.dirname(path.dirname(dirname));

const run = async (command: string, args: (string)[], options: SpawnOptionsWithoutStdio) => {
    return new Promise<void>((resolve, reject) => {
        const proc = child.spawn(command, args, options);
        proc.stdout.on("data", (data) => {
            console.info(String(data).trim());
        });

        proc.stderr.on("data", (data) => {
            console.info(String(data).trim());
        });

        proc.on("error", (error) => {
            reject(error);
        });

        proc.on("close", (code) => {
            if (!code) {
                resolve();
                return;
            }
            reject(new Error(`${command} ${args.join(" ")} exited with code ${code}`));
        });
    });
};

const PKG_JSON_INDENT = 2;
const JSON_INDENT = 4;

interface Data { [key: string]: unknown }

interface Options {
    targetDir: string;
    pkg: PackageJson;
}

type GetSetPath = string | string[];

type ContentsUpdater = (fileContents: string) => (Promise<string> | string);
type JSONContentsUpdater<D = Data> = (
    fileData: D, setFn: (objPath: GetSetPath, value: unknown) => void
) => (Promise<D | undefined> | D | undefined);

type Dep = "dependencies" | "devDependencies";

class Migration {
    private readonly _targetDir: string;

    private readonly _pkg: PackageJson;

    public constructor(opts: Options) {
        this._targetDir = opts.targetDir;
        this._pkg = opts.pkg;
    }

    private _savePkg() {
        return fs.writeFile(
            join(this._targetDir, "package.json"),
            JSON.stringify(this._pkg, null, PKG_JSON_INDENT),
        );
    }

    public get jsx() {
        return Boolean(this._pkg.libraryTemplate!.jsx);
    }

    public get targetDir() {
        return this._targetDir;
    }

    public get pkg() {
        return this._pkg;
    }

    public async setPath(objPath: GetSetPath, value: unknown) {
        set(this._pkg, objPath, value);
        await this._savePkg();
    }

    public async setScript(scriptName: string, value: string) {
        await this.setPath(["scripts", scriptName], value);
    }

    public async deletePath(objPath: GetSetPath) {
        const thePath = typeof objPath === "string" ? objPath.split(".") : [...objPath];
        const last = thePath.pop();

        const item = get(this._pkg, thePath);
        if (item == null || !last) {
            return;
        }

        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete (item as { [key: string]: unknown })[last];
        await this._savePkg();
    }

    public async deleteScript(scriptName: string) {
        await this.deletePath(["scripts", scriptName]);
    }

    public async upgradeScript(scriptName: string, oldValue: string, newValue: string) {
        this.assertScript(
            scriptName,
            oldValue,
            new Error(
                "cannot update " + scriptName + " script as it was modified\n  "
                + "wanted new value:\n  `" + newValue + "`",
            ),
        );
        await this.setScript(scriptName, newValue);
    }

    public async addDependency(name: string, version: string) {
        await this.setPath(["dependencies", name], version);
    }

    public async addDevDependency(name: string, version: string) {
        await this.setPath(["devDependencies", name], version);
    }

    public async upgradeDependency(name: string, version: string, defaultType: Dep = "dependencies") {
        const type = this.findDependency(name) ?? defaultType;
        await this.setPath([type, name], version);
    }

    public findDependency(name: string) {
        const deps = this._pkg.dependencies ?? {};
        const dev = this._pkg.devDependencies ?? {};
        if (deps[name]) {
            return "dependencies";
        }
        if (dev[name]) {
            return "devDependencies";
        }
    }

    public async remove(dirName: string) {
        await fs.remove(join(this._targetDir, dirName));
    }

    public async copy(sourceName: string, targetName: string | null = null, overwrite = true) {
        await fs.copy(
            join(thisDir, sourceName),
            join(this._targetDir, targetName ?? sourceName),
            { overwrite },
        );
    }

    public async rename(source: string, target: string) {
        await fs.rename(join(this._targetDir, source), join(this._targetDir, target));
    }

    public async updateContents(file: string, updater: ContentsUpdater) {
        const target = join(this._targetDir, file);
        const data = String(await fs.readFile(target));
        const contents = await updater(data);
        await fs.writeFile(target, contents);
    }

    public async updateContentsJSON<D = Data>(file: string, updater: JSONContentsUpdater<D>) {
        const target = join(this._targetDir, file);
        const data = JSON.parse(String(await fs.readFile(target))) as D;
        const newData = await updater(
            data,
            // @ts-expect-error `set` needs better typings
            set.bind(null, data),
        );
        await fs.writeFile(target, JSON.stringify(newData ?? data, null, JSON_INDENT));
    }

    public async updatePath(objPath: GetSetPath, updater: ContentsUpdater) {
        const current = get(this._pkg, objPath);
        const newValue = updater(String(current));
        await this.setPath(objPath, newValue);
    }

    public assertPath(objPath: GetSetPath, value: unknown, error: Error) {
        if (get(this._pkg, objPath) !== value) {
            throw error;
        }
    }

    public assertNoPath(objPath: GetSetPath, error: Error) {
        if (get(this._pkg, objPath) !== undefined) {
            throw error;
        }
    }

    public assertScript(scriptName: string, value: string, error: Error) {
        this.assertPath(["scripts", scriptName], value, error);
    }

    public assertNoScript(scriptName: string, error: Error) {
        this.assertNoPath(["scripts", scriptName], error);
    }

    public assertNoDependency(name: string, error: Error) {
        if (this._pkg.dependencies && name in this._pkg.dependencies) {
            throw error;
        }
    }

    public assertNoDevDependency(name: string, error: Error) {
        if (this._pkg.devDependencies && name in this._pkg.devDependencies) {
            throw error;
        }
    }

    public assertNoAnyDependency(name: string, error: Error) {
        this.assertNoDependency(name, error);
        this.assertNoDevDependency(name, error);
    }

    public assertDevDependency(name: string, version: string | null, error: Error) {
        if (
            !this._pkg.devDependencies
            || (
                (version && this._pkg.devDependencies[name] !== version)
                || (!version && !this._pkg.devDependencies[name])
            )
        ) {
            throw error;
        }
    }

    public async removeDependency(name: string) {
        await this.deletePath(["dependencies", name]);
    }

    public async removeDevDependency(name: string) {
        await this.deletePath(["devDependencies", name]);
    }

    public async removeAnyDependency(name: string) {
        const depType = this.findDependency(name);
        if (!depType) { return; }
        await this.deletePath([depType, name]);
    }

    public yarn() {
        return run("yarn", [], {
            cwd: this._targetDir,
            shell: true,
        });
    }
}

export { Migration };
export type { PackageJson };
